==> netams-3.1.1830.tar.gz: netams-3.1.1830/src/netamsctl.c <==
/*************************************************************************
***     Authentication, authorization, accounting + firewalling package
***     (c) 1998-2003 Anton Vinokurov <anton@netams.com>
***		(c) 2003 NeTAMS Development Team
***		All rights reserved. See 'Copying' file included in distribution
***		For latest version and more info, visit this project web page
***		located at http://www.netams.com
***		 					
*************************************************************************/
/* $Id: netamsctl.c,v 1.6.2.3 2004/05/06 10:35:03 jura Exp $ */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

int main(int argc, char **argv){
int op;
char *rcfile=NULL;
char *buffer;
FILE *RCFILE;
int i;

char *login=NULL, *password=NULL, *host=NULL, *cmd=NULL;
unsigned port=20001;

cmd=(char*)malloc(1024); bzero(cmd, 1023);

while((op=getopt(argc, argv, "f:")) != EOF){
	switch(op){
		case 'f': rcfile=(char *)malloc(strlen(optarg)+1);
			  strcpy(rcfile, optarg);
			  break;
		default: break;
		}
	}

for (i=optind; i<argc; i++) {
	strncat(cmd, argv[i], 255);
	strncat(cmd, " ", 5);
	}

if (rcfile) {
	RCFILE=fopen(rcfile, "rt");
	if (!RCFILE) { printf("unable to open specified .rc file, %s\n", rcfile); exit(-1); }
	}
else {
	RCFILE=fopen("~/.netamsctl.rc", "rt");
	if (!RCFILE) RCFILE=fopen(".netamsctl.rc", "rt");
	if (!RCFILE) RCFILE=fopen("/usr/local/etc/.netamsctl.rc", "rt");
	if (!RCFILE) RCFILE=fopen("/etc/.netamsctl.rc", "rt");
	if (!RCFILE) { printf("unable to open .rc file\n"); exit(-1); }
	}

buffer=(char *)malloc(4*1024+1); 
while (!feof(RCFILE)){
	fgets(buffer, 4*1024, RCFILE);
	i=strlen(buffer);
	buffer[i-1]=0; //thus we remove \n
	
	if (!strncasecmp(buffer, "login=", 6)) { 
		i-=6;
		login=(char*)malloc(i);
		strncpy(login, buffer+6, i-1);
		login[i-1]=0;
		} 

	else if (!strncasecmp(buffer, "password=", 9)) { 
		i-=9;
		password=(char*)malloc(i);
		strncpy(password, buffer+9, i-1);
		password[i-1]=0;
		} 

	else if (!strncasecmp(buffer, "host=", 5)) { 
		i-=5;
		host=(char*)malloc(i);
		strncpy(host, buffer+5, i-1);
		host[i-1]=0;
		} 

	else if (!strncasecmp(buffer, "port=", 5)) { 
		port=atoi(buffer+5);
		} 

	bzero(buffer, 4*1024);
	}  

fclose(RCFILE);

if (!login || !password) { printf ("login or password unspecified\n"); exit(-1); }
if (host==NULL) {
	host=(char*)malloc(i=strlen("localhost")+1);
	strncpy(host, "localhost", i-1);
	}


//printf("host: %s port %u login: %s password: %s\n", host, port, login, password);
//printf("cmd: %s\n", cmd);
if (cmd[0]=='\0') { printf("no command is given\n"); exit(-1); }

int sock;
struct sockaddr_in addr;
struct hostent *hp;

if ((sock=socket(AF_INET,SOCK_STREAM,0))==-1 ) { printf("Connect failed. Possibly NeTAMS is not running?\n"); exit(-1); }

bzero((unsigned char *)(&addr), sizeof(addr));
addr.sin_family=AF_INET;
hp=gethostbyname(host);
if(!hp) {
	printf("gethostbyname failed\n");
	exit(-1);
	}
memcpy((char*)&addr.sin_addr, hp->h_addr_list[0], hp->h_length);
addr.sin_port=htons((unsigned short)(port));

if ((i=connect(sock, (struct sockaddr*)&addr, sizeof(addr)))) { printf("Connect failed. Possibly NeTAMS is not running?\n"); exit(-1); }

FILE *fd;

fd = fdopen(sock, "w+");
setvbuf(fd, NULL, _IONBF, 0);
if (fd==NULL) { printf("fdopen of server socket() failed\n"); exit(-1); } 

fgets(buffer, 4*1024, fd); // printf("got: %s", buffer); 
fgets(buffer, 4*1024, fd); // printf("got: %s", buffer); 

bzero(buffer, 1023); fread(buffer, strlen("NeTAMS user login >"), 1, fd); 
// printf("got: %s", buffer);

fprintf(fd, "%s \n", login);	fflush(fd);								 

i=strlen("NeTAMS user password >")+strlen(login);
bzero(buffer, 1023); fread(buffer, 19, 1, fd);
if (buffer[18]=='>') { printf("login: failed\n"); shutdown(sock, SHUT_RDWR); close(sock); exit(-1); }
else { bzero(buffer, 1023); fread(buffer, i-18, 1, fd); } 
//printf("got: %s", buffer);

fprintf(fd, "%s \n", password); fflush(fd);

fgets(buffer, 4*1024, fd); // printf("got: %s", buffer);
fgets(buffer, 4*1024, fd); // printf("got: %s", buffer);
fgets(buffer, 4*1024, fd); // printf("got: %s", buffer);
bzero(buffer, 1023); fread(buffer, 1, 1, fd); 
if (buffer[0]!='>') { printf("login: failed\n"); shutdown(sock, SHUT_RDWR); close(sock); exit(-1); }
else ; //printf("login: ok\n"); 

bzero(buffer, 1023);

fprintf(fd, "%s\n", cmd);

int prev=10;
while (1) {
	i=fgetc(fd);
	if (feof(fd) ||(i=='>' && (prev==10 || prev==13))) break;
	putchar(i);
	prev=i;
	}

shutdown(sock, SHUT_RDWR);
close(sock); 

free(buffer);

return 0;
}
/////////////////////////////////////////////////////////////

==> netams-3.2.2.tar.gz: netams-3.2.2/src/netamsctl.c <==
/*************************************************************************
***	Authentication, authorization, accounting + firewalling package
***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
***	(c) 2002-2005 NeTAMS Development Team
***	All rights reserved. See 'Copying' file included in distribution
***	For latest version and more info, visit this project web page
***	located at http://www.netams.com
***
*************************************************************************/
/* $Id: netamsctl.c,v 1.19 2005/01/17 13:13:21 jura Exp $ */

#include "config.h"

int main(int argc, char **argv){
int op;
char *rcfile=NULL;
char *buffer;
FILE *RCFILE=NULL;
int i;
char *login=NULL, *password=NULL, *host=NULL, *cmd=NULL;
unsigned short port=20001;

cmd=(char*)malloc(1024); bzero(cmd, 1023);

while((op=getopt(argc, argv, "f:")) != EOF){
	switch(op){
		case 'f': rcfile=(char *)malloc(strlen(optarg)+1);
				  strcpy(rcfile, optarg);
				  break;
		default: break;
		}
	}

for (i=optind; i<argc; i++) {
	strncat(cmd, argv[i], 255);
	strncat(cmd, " ", 5);
	}

if (rcfile) {
	RCFILE=fopen(rcfile, "rt");
	if (!RCFILE) { printf("unable to open specified .rc file, %s\n", rcfile); exit(-1); }
	}
else {
	char *homedir=getenv("HOME");
	if (homedir!=NULL) {
		char *str=(char*)malloc(256);
		sprintf(str, "%s/.netamsctl.rc", homedir);
		RCFILE=fopen(str, "rt");
	}
	if (!RCFILE) RCFILE=fopen(".netamsctl.rc", "rt");
	if (!RCFILE) RCFILE=fopen("/usr/local/etc/.netamsctl.rc", "rt");
	if (!RCFILE) RCFILE=fopen("/etc/.netamsctl.rc", "rt");
	if (!RCFILE) { printf("unable to open .rc file\n"); exit(-1); }
	}

buffer=(char *)malloc(4*1024+1); 
while (!feof(RCFILE)){
	fgets(buffer, 4*1024, RCFILE);
	i=strlen(buffer);
	buffer[i-1]=0; //thus we remove \n
	
	if (!strncasecmp(buffer, "login=", 6)) { 
		i-=6;
		login=(char*)malloc(i+1);
		strncpy(login, buffer+6, i-1);
		login[i-1]=0;
		} 

	else if (!strncasecmp(buffer, "password=", 9)) { 
		i-=9;
		password=(char*)malloc(i+1);
		strncpy(password, buffer+9, i-1);
		password[i-1]=0;
		} 

	else if (!strncasecmp(buffer, "host=", 5)) { 
		i-=5;
		host=(char*)malloc(i+1);
		strncpy(host, buffer+5, i-1);
		host[i-1]=0;
		} 

	else if (!strncasecmp(buffer, "port=", 5)) { 
		port=strtol(buffer+5, NULL, 10);
		} 

	bzero(buffer, 4*1024);
	}  

fclose(RCFILE);

if (!login || !password) { printf ("login or password unspecified\n"); exit(-1); }
if (host==NULL) {
	host=(char*)malloc(i=strlen("localhost")+1);
	strncpy(host, "localhost", i-1);
	}


printf("host: %s port: %u login: %s password: %s\n", host, port, login, password);
printf("cmd: %s\n", cmd);
if (cmd[0]=='\0') { printf("no command is given\n"); exit(-1); }

int sock;
struct sockaddr_in addr;
struct hostent *hp;

if ((sock=socket(AF_INET,SOCK_STREAM,0))==-1 ) { printf("Connect failed. Possibly NeTAMS is not running?\n"); exit(-1); }

bzero(&addr, sizeof(addr));
addr.sin_family=AF_INET;
hp=gethostbyname(host);
if(!hp) {
	herror("gethostbyname failed ");
	exit(-1);
	}
memcpy((char*)&addr.sin_addr, hp->h_addr_list[0], hp->h_length);
addr.sin_port=htons((unsigned short)(port));

if ((i=connect(sock, (struct sockaddr*)&addr, sizeof(addr)))) { printf("Connect failed. Possibly NeTAMS is not running?\n"); exit(-1); }

FILE *fd;

fd = fdopen(sock, "w+");
setvbuf(fd, NULL, _IONBF, 0);
if (fd==NULL) { printf("fdopen of server socket() failed\n"); exit(-1); } 

fgets(buffer, 4*1024, fd); // printf("got: %s", buffer); 
fgets(buffer, 4*1024, fd); // printf("got: %s", buffer); 

bzero(buffer, 1023); fread(buffer, strlen("NeTAMS user login >"), 1, fd); 
// printf("got: %s", buffer);

fprintf(fd, "%s \n", login);	fflush(fd);								 

i=strlen("NeTAMS user password >")+strlen(login);
bzero(buffer, 1023); fread(buffer, 19, 1, fd);
if (buffer[18]=='>') { printf("login: failed\n"); shutdown(sock, SHUT_RDWR); close(sock); exit(-1); }
else { bzero(buffer, 1023); fread(buffer, i-18, 1, fd); } 
//printf("got: %s", buffer);

fprintf(fd, "%s \n", password); fflush(fd);

fgets(buffer, 4*1024, fd); // printf("got: %s", buffer);
fgets(buffer, 4*1024, fd); // printf("got: %s", buffer);
fgets(buffer, 4*1024, fd); // printf("got: %s", buffer);
bzero(buffer, 1023); fread(buffer, 1, 1, fd); 
if (buffer[0]!='>') { printf("login: failed\n"); shutdown(sock, SHUT_RDWR); close(sock); exit(-1); }
else ; //printf("login: ok\n"); 

bzero(buffer, 1023);

char *p=cmd, *pp;
do {
	pp=strstr(p, "&&");
	if (pp) *pp='\0';
	fprintf(fd, "%s\n", p);
	p=pp+3;
	
	int prev=10;
	while (1) {
		i=fgetc(fd);
		if (feof(fd) || (i=='#' && (prev!=10 && prev!=13) ) || (i=='>' && (prev==10 || prev==13))) break;
		putchar(i);
		prev=i;
	}
} while (pp);

shutdown(sock, SHUT_RDWR);
close(sock); 

free(buffer);

return 0;
}
/////////////////////////////////////////////////////////////
